import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Vector;

public class Singstar extends Frame implements WindowListener {

	private static final long serialVersionUID = 938731515102246428L;

	public static final double SAMPLE_RATE = 44100.0;
	public static final double SECONDS_PER_SAMPLE = 1 / SAMPLE_RATE;
	public static final double HALFTONE_RATIO = 1.0594630943592952645618252949463; // 2 ^ 1/12
	public static final double TWO_HALFTONES_RATIO = HALFTONE_RATIO * HALFTONE_RATIO;
	public static final double FOUR_HALFTONES_RATIO = TWO_HALFTONES_RATIO * TWO_HALFTONES_RATIO;
	public static final double EIGHT_HALFTONES_RATIO = FOUR_HALFTONES_RATIO * FOUR_HALFTONES_RATIO;
	public static final int HALFTONES_PER_OCTAVE = 12;
	public static final double BASE_FREQUENCY = 55.0;
	
	private static final int[] bufferLengthList = {
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.0000000000000000) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.0594630943592953) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.1224620483093730) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.1892071150027210) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.2599210498948732) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.3348398541700344) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.4142135623730951) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.4983070768766815) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.5874010519681994) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.6817928305074292) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.7817974362806788) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 1.8877486253633870) + 0.5),
			(int) (SAMPLE_RATE / (BASE_FREQUENCY * 2.0000000000000000) + 0.5)
			};

	private double[] sampleList;
	private Image screenBuffer;
	private OverlayBuffer[] bufferList;
	
	public static void main(String[] args) {
		new Singstar();
	}
	
	public Singstar()
	{
		bufferList = new OverlayBuffer[bufferLengthList.length];
		for (int i = 0; i < bufferLengthList.length; i++)
		{
			bufferList[i] = new OverlayBuffer(bufferLengthList[i]);
		}

/*		
		sampleList = new double[(int) SAMPLE_RATE];
		double f = BASE_FREQUENCY * 1.4142135623730950488016887242097 * 2; // Testfrequenz
		double amp = 0.5;
		for (int i = 0; i < sampleList.length / 2; i++)
		{
			double time = i * SECONDS_PER_SAMPLE;
			double a = f * time * 2 * Math.PI;
			sampleList[i] = amp * (Math.sin(a + 1.1) / 1 + Math.cos(a * 2 - 1.2) / 2 + Math.sin(a * 3 - 1.3) / 3 + Math.cos(a * 4 + 1.4) / 4);
		}

		for (int i = sampleList.length / 2; i < sampleList.length; i++)
		{
			double time = i * SECONDS_PER_SAMPLE;
			double a = f * time * 2 * Math.PI * HALFTONE_RATIO * HALFTONE_RATIO * HALFTONE_RATIO;
			sampleList[i] = amp * (Math.sin(a + 1.1) / 1 + Math.cos(a * 2 - 1.2) / 2 + Math.sin(a * 3 - 1.3) / 3 + Math.cos(a * 4 + 1.4) / 4);
		}
*/
		
		byte[] dataList = null;

		try
		{
			File file = new File("a.wav");
			FileInputStream file_input = new FileInputStream (file);
			file_input.skip(58);
			dataList = new byte[(int) file.length() - 58];
			file_input.read(dataList, 0, dataList.length);
			file_input.close();
		}
		catch  (IOException e)
		{
			System.out.println ( "IO Exception =: " + e );
		}		

		sampleList = new double[dataList.length];
		double last = 0.0;
		for (int i = 0 ; i < dataList.length; i++)
		{
			sampleList[i] = last * 0.95 + ((dataList[i] + 256) % 256 - 128) / 128.0 * 0.05;
			last = sampleList[i];
		}

		int density = (int) (sampleList.length / 800) + 1;
		
		for (int i = 0; i < sampleList.length; i++)
		{
			for (int j = 0; j < bufferList.length; j++)
			{
				bufferList[j].addSample(sampleList[i]);
			}
			
			if (i % density == 0)
			{
				for (int j = 0; j < bufferList.length; j++)
				{
					bufferList[j].measure();
				}
			}
		}

		this.setTitle("singstar Notensuche");
		this.setSize(1200, 900);
		this.addWindowListener(this);
		this.setBackground(Color.BLACK);
		this.setForeground(Color.WHITE);
		this.setVisible(true);
	}
	
	public void paint(Graphics g)
	{
		if (this.screenBuffer != null)
		{
			Insets insets = this.getInsets();
			g.drawImage(this.screenBuffer, insets.left, insets.top, null);
		}
	}
	

	public void windowClosing(WindowEvent e) {
		this.dispose();
	}

	public void windowDeactivated(WindowEvent e) {	}
	public void windowDeiconified(WindowEvent e) {	}
	public void windowIconified(WindowEvent e) {	}
	public void windowOpened(WindowEvent e) {
		Insets insets = this.getInsets();
		int screenBufferWidth = this.getWidth() - insets.right - insets.left;
		int screenBufferHeight = this.getHeight() - insets.bottom - insets.top;
		screenBuffer = this.createImage(screenBufferWidth, screenBufferHeight);

		Graphics g = screenBuffer.getGraphics();

		for (int j = 0; j < bufferList.length; j++)
		{
			bufferList[j].draw(g, bufferLengthList[0], j * 64);
		}

	}
	public void windowActivated(WindowEvent e) {	}
	public void windowClosed(WindowEvent e) {	}

}

class OverlayBuffer
{

	private static final double STRENGTH = 0.25;
	
	private double[] buffer;
	private int index = 0;
	public double squareSum = 0.0;
	private Vector<Double> squareSumList = new Vector<Double>();
	
	public OverlayBuffer(int size)
	{
		buffer = new double[size];
	}
	
	public void addSample(double sample)
	{
		squareSum -= buffer[index] * buffer[index];
		buffer[index] = (buffer[index] * (1.0 - STRENGTH)) + (sample * STRENGTH);
		squareSum += buffer[index] * buffer[index];
		index = (index + 1) % buffer.length;
	}
	
	public void measure()
	{
		squareSumList.add(new Double(64 * Math.sqrt(2 * squareSum / buffer.length)));
	}
	
	public void draw(Graphics g, int x, int y)
	{
		g.setColor(Color.GRAY);
		g.drawLine(0, y + 32, 300, y + 32);

		g.setColor(Color.LIGHT_GRAY);
		g.drawRect(0, y, 300, 64);
		
		int xPoints[] = new int[buffer.length];
		int yPoints[] = new int[buffer.length];
		
		for (int i = 1; i < buffer.length; i++)
		{
			xPoints[i] = (int) (300.0 * i / buffer.length);
			yPoints[i] = y + 32 - (int) (32 * buffer[i]);
		}
		g.setColor(Color.YELLOW);
		g.drawPolyline(xPoints, yPoints, buffer.length);

		for (int i = 1; i < squareSumList.size(); i++)
		{
			g.setColor(Color.GREEN);
			g.drawLine(310 + i, y + 64, 310 + i, y + 64 - squareSumList.get(i).intValue());
		}
		
	}
	
}